@@ -1,14 +1,3 @@
-# /// script
-# requires-python = ">=3.9"
-# dependencies = [
-#     "anthropic",
-# ]
-#
-# [tool.uv.sources]
-# anthropic = { path = "../", editable = true }
-# ///
-
-
 import pydantic
 
 import anthropic
@@ -27,7 +16,7 @@ Extract the product name, price, and quantity from this customer message:
 "Hi, I’d like to order 2 packs of Green Tea for 5.50 dollars each."
 """
 
-parsed_message = client.beta.messages.parse(
+parsed_message = client.messages.parse(
     model="claude-sonnet-4-5",
     messages=[{"role": "user", "content": prompt}],
     max_tokens=1024,
