@@ -6,7 +6,7 @@ client = Anthropic()
 
 # Create a message with web search enabled
 message = client.messages.create(
-    model="claude-3-5-sonnet-latest",
+    model="claude-sonnet-4-5-20250929",
     max_tokens=1024,
     messages=[{"role": "user", "content": "What's the weather in New York?"}],
     tools=[
